@@ -58,6 +58,16 @@ class SCCIClientError(SCCIError):
         super(SCCIClientError, self).__init__(message)
 
 
+class SCCIRAIDNotReady(SCCIError):
+    """SCCIRAIDNotReady
+
+    This exception is used when a mechanism not applied
+    into a configuration on the iRMC yet
+    """
+    def __init__(self, message):
+        super(SCCIRAIDNotReady, self).__init__(message)
+
+
 """
 List of iRMC S4 supported SCCI commands
 
@@ -492,7 +502,8 @@ def get_essential_properties(report, prop_keys):
 
 def get_capabilities_properties(d_info,
                                 capa_keys,
-                                pci_device_ids,
+                                gpu_ids,
+                                fpga_ids=None,
                                 **kwargs):
     """get capabilities properties
 
@@ -502,8 +513,10 @@ def get_capabilities_properties(d_info,
 
     :param d_info: the dictionary of ipmitool parameters for accessing a node.
     :param capa_keys: a list of keys for additional capabilities properties.
-    :param pci_device_ids: the list of string contains <vendorID>/<deviceID>
+    :param gpu_ids: the list of string contains <vendorID>/<deviceID>
     for GPU.
+    :param fpga_ids: the list of string contains <vendorID>/<deviceID>
+    for CPU FPGA.
     :param kwargs: additional arguments passed to scciclient.
     :returns: a dictionary which contains keys and their values.
     """
@@ -529,11 +542,15 @@ def get_capabilities_properties(d_info,
         # Sometime the server started but PCI device list building is
         # still in progress so system will response error. We have to wait
         # for some more seconds.
-        if kwargs.get('sleep_flag', False) and 'pci_gpu_devices' in capa_keys:
+        if kwargs.get('sleep_flag', False) and \
+                any(k in capa_keys for k in ('pci_gpu_devices', 'cpu_fpga')):
             time.sleep(5)
 
         if 'pci_gpu_devices' in capa_keys:
-            v['pci_gpu_devices'] = ipmi.get_gpu(d_info, pci_device_ids)
+            v['pci_gpu_devices'] = ipmi.get_pci_device(d_info, gpu_ids)
+
+        if fpga_ids is not None and 'cpu_fpga' in capa_keys:
+            v['cpu_fpga'] = ipmi.get_pci_device(d_info, fpga_ids)
 
         if 'trusted_boot' in capa_keys:
             v['trusted_boot'] = ipmi.get_tpm_status(d_info)
@@ -541,3 +558,35 @@ def get_capabilities_properties(d_info,
         return v
     except (snmp.SNMPFailure, ipmi.IPMIFailure) as err:
         raise SCCIClientError('Capabilities inspection failed: %s' % err)
+
+
+def get_raid_fgi_status(report):
+    """Gather fgi(foreground initialization) information of raid configuration
+
+    This function returns a fgi status which contains activity status
+    and its values from the report.
+
+    :param report: SCCI report information
+    :returns: dict of fgi status of logical_drives, such as Initializing (10%)
+              or Idle. e.g: {'0': 'Idle', '1': 'Initializing (10%)'}
+    :raises: SCCIInvalidInputError: fail report input.
+             SCCIRAIDNotReady: waiting for RAID configuration to complete.
+    """
+    fgi_status = {}
+    raid_path = "./Software/ServerView/ServerViewRaid"
+
+    if not report.find(raid_path):
+        raise SCCIInvalidInputError(
+            "ServerView RAID not available in Bare metal Server")
+    if not report.find(raid_path + "/amEMSV/System/Adapter/LogicalDrive"):
+        raise SCCIRAIDNotReady(
+            "RAID configuration not configure in Bare metal Server yet")
+
+    logical_drives = report.findall(raid_path +
+                                    "/amEMSV/System/Adapter/LogicalDrive")
+    for logical_drive_name in logical_drives:
+        status = logical_drive_name.find("./Activity").text
+        name = logical_drive_name.find("./LogDriveNumber").text
+        fgi_status.update({name: status})
+
+    return fgi_status
